
local PoleFlag = ...

-- Create default building part registering function 
function registerDefaultBuildingPart(_nodePrefix)
	PoleFlag:register({
		DataType = "BUILDING_PART",
		Id = _nodePrefix .. "_PART",
		Mover = { DataType = "BUILDING_PART_MOVER" },
		ConstructorData = {
                     DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
                     CoreObjectPrefab = _nodePrefix .. "_PREFAB"
                }
	})
end

function registerColor(_colorFbxName, _colorAssetId)

    PoleFlag:registerAssetProcessor("models/flags/poleFlag" .. _colorFbxName .. ".fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
    })

-- Register common prefab nodes
    PoleFlag:registerAssetId("models/flags/poleFlag" .. _colorFbxName .. ".fbx/Prefab/poleFlag/Pole/step_0_RootPart", "SGV3_POLE_FLAG" .. _colorAssetId .. "_ROOT_PREFAB")
    PoleFlag:registerAssetId("models/flags/poleFlag" .. _colorFbxName .. ".fbx/Prefab/poleFlag/Pole/step_3_TopPart", "SGV3_POLE_FLAG" .. _colorAssetId .. "_TOP_PREFAB")
    PoleFlag:registerAssetId("models/flags/poleFlag" .. _colorFbxName .. ".fbx/Prefab/poleFlag/Pole/step_2_Tiling2Part", "SGV3_POLE_FLAG" .. _colorAssetId .. "_TILING2_PREFAB")
    PoleFlag:registerAssetId("models/flags/poleFlag" .. _colorFbxName .. ".fbx/Prefab/poleFlag/Pole/step_1_Tiling3Part", "SGV3_POLE_FLAG" .. _colorAssetId .. "_TILING3_PREFAB")

    local defaultNodePrefixList = {
	"SGV3_POLE_FLAG" .. _colorAssetId .. "_TOP",
	"SGV3_POLE_FLAG" .. _colorAssetId .. "_TILING2",
	"SGV3_POLE_FLAG" .. _colorAssetId .. "_TILING3"
    }

-- Register simple building part assets
    for i, nodePrefix in ipairs(defaultNodePrefixList) do
	registerDefaultBuildingPart(nodePrefix)
    end

-- Register the pole flag part
    PoleFlag:register({
	DataType = "BUILDING_PART",
	Id = "SGV3_POLE_FLAG" .. _colorAssetId .. "_ROOT_PART",
	Name = "SGV3_POLE_FLAG" .. _colorAssetId .. "_ROOT_NAME",
	Description = "SGV3_POLE_FLAG" .. _colorAssetId .. "_ROOT_DESC",
	Category = "DECORATION",
	Mover = { DataType = "BUILDING_PART_MOVER_INSTANCE" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_SCALER", -- Since there are different types of building constructor, 'DataType' is necessary here
		CoreObjectPrefab = "SGV3_POLE_FLAG" .. _colorAssetId .. "_ROOT_PREFAB",
		EndPart = "SGV3_POLE_FLAG" .. _colorAssetId .. "_TOP_PART",
		FillerList = {
			"SGV3_POLE_FLAG" .. _colorAssetId .. "_TILING2_PART"
                },
		BasementFillerList = {
			"SGV3_POLE_FLAG" .. _colorAssetId .. "_TILING3_PART"
         	},
		MinimumScale = 0,
		BasementScale = 1,
		IsVertical = true
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
                        { Resource = "WOOD", Quantity = 2 },
			{ Resource = "WOOLEN_CLOTH", Quantity = 2 }
		}
	}
    })

-- Override building part components properties (generated by the building asset processor) to add building zones

    PoleFlag:registerPrefabComponent("models/flags/poleFlag" .. _colorFbxName .. ".fbx/Prefab/poleFlag/Pole/step_0_RootPart", {
	DataType = "COMP_BUILDING_PART",
	BuildingPartType = "MINOR", -- Snaps the decoration only on minor attach nodes (here only on the roofs)
	BuildingZone = { 1, 1 }
    })

-- Particle system
    PoleFlag:registerAssetId("models/flags/Flag" .. _colorFbxName .. ".png", "SGV3_FLAG" .. _colorAssetId .. "_TEXTURE")

    PoleFlag:register({
	DataType = "MATERIAL",
	Id = "SGV3_FLAG" .. _colorAssetId .. "_MATERIAL",
        AlbedoColor = COL_WHITE,
	AlbedoTexture = "SGV3_FLAG" .. _colorAssetId .. "_TEXTURE"
    })

    PoleFlag:register({
	DataType = "PARTICLE_SYSTEM",
	Id = "SGV3_FLAG" .. _colorAssetId .. "_PARTICLE_SYSTEM",
	Material = "SGV3_FLAG" .. _colorAssetId .. "_MATERIAL",
	BillboardBehavior = "FACE_CAMERA_Y_ALIGN",
	Duration = 1,
	Looping = true,
	Delay = { 0, 0 },
	LifeTime = 1,
	StartSpeed = { 0, 0 },
	StartSize = { 10, 10 },
	StartRotation = { 0, 0 },
	StartColor = { 1, 1, 1, 1 },
	GravityModifier = 0,
	RateOverTime = 2,
	BurstList = {},
	Shape = {
		DataType = "PARTICLE_EMITTER_SHAPE_BOX",
		Size = { 0, 0.001, 0 },
                Rotation = { 0, 0, 0 },
                Position = { 0, 0, 0 }
	},
	LinearVelocity = {
		DataType = "PARTICLE_FLOAT3_VALUE_CONSTANT_RANDOM",
		ValueA = { 0, 0, 0 },
		ValueB = { 0, 0, 0 },
	},
--	ColorOverLifeTime = {
--		DataType = "GRADIENT",
--		ColorList = {
--			{ T = 0, Color = { 0, 0, 0, 0} }
--,
--			{ T = 0.3, Color = { 0, 0, 0, 0} },
--			{ T = 0.7, Color = { 0, 0, 0, 0} },
--			{ T = 1, Color = { 0, 0, 0, 0 } }
--		},
--		AlphaList = {
--			{ T = 0, Alpha = 255 }
--,
--			{ T = 0.3, Alpha = 255 },
--			{ T = 0.7, Alpha = 255 },
--			{ T = 1, Alpha = 0 }
--		},
--	},
	SizeOverLifetime = {
		{ T = 0, Value = 1 }
--,
--		{ T = 0.7, Value = 0.5 },
--		{ T = 1, Value = 0.5 }
	},
	RotationOverLifetime = { 0, 0 },
    })

-- Register air vent particle system (drapeau on octogonal tower)
    PoleFlag:registerAssetId("models/flags/poleFlag" .. _colorFbxName .. ".fbx/Prefab/poleFlag/Pole/step_3_TopPart/Flag" .. _colorFbxName .. "Emitter", "SGV3_FLAG" .. _colorAssetId .. "_EMITTER_PREFAB")

    PoleFlag:registerPrefabComponent("models/flags/poleFlag" .. _colorFbxName .. ".fbx/Prefab/poleFlag/Pole/step_3_TopPart", {
	DataType = "COMP_BUILDING_PART",
	FeedbackComponentListToActivate = {
                 { "SGV3_FLAG" .. _colorAssetId .. "_EMITTER_PREFAB", "COMP_PARTICLE_EMITTER_TOGGLE" }
	}
    })

    PoleFlag:registerPrefabComponent("models/flags/poleFlag" .. _colorFbxName .. ".fbx/Prefab/poleFlag/Pole/step_3_TopPart/Flag" .. _colorFbxName .. "Emitter", {
	DataType = "COMP_PARTICLE_EMITTER",
	ParticleSystem = "SGV3_FLAG" .. _colorAssetId .. "_PARTICLE_SYSTEM",
	IsPlaying = true,
	IsEmitting = false
    })

    PoleFlag:registerPrefabComponent("models/flags/poleFlag" .. _colorFbxName .. ".fbx/Prefab/poleFlag/Pole/step_3_TopPart/Flag" .. _colorFbxName .. "Emitter", {
	DataType = "COMP_PARTICLE_EMITTER_TOGGLE",
	Enabled = false
    })
end

registerColor("Red", "_RED")
registerColor("Blue", "_BLUE")
registerColor("Green", "_GREEN")
registerColor("Purple", "_PURPLE")
registerColor("Orange", "_ORANGE")
registerColor("Turquoise", "_TURQUOISE")
--------------------------------------------------------------------------
